@@ -74,11 +74,13 @@ int main(int argc,char **argv)
 		gr.Clf();			// make new drawing
 		gr.Line(mglPoint(),pnt,"Ar2");
 		char str[10] = "i=0";	str[3] = '0'+i;
+		gr->Puts(mglPoint(),str);
 		gr.Update();		// update window
 	}
 	return 0;	// finish calculations and close the window
 }
 #else		// just default samples
+//-----------------------------------------------------------------------------
 int test_wnd(mglGraph *gr);
 int sample(mglGraph *gr);
 int sample_1(mglGraph *gr);
@@ -86,12 +88,41 @@ int sample_2(mglGraph *gr);
 int sample_3(mglGraph *gr);
 int sample_d(mglGraph *gr);
 //-----------------------------------------------------------------------------
+#if MGL_HAVE_PTHR_WIDGET
+class myDraw : public mglDraw
+{
+	mglPoint pnt;	// some variable for changeable data
+	long i;			// another variable to be shown
+	mglWnd *wnd;	// external window for plotting
+public:
+	myDraw(mglWnd *w=0) : mglDraw()	{	wnd=w;	}
+	void SetWnd(mglWnd *w)	{	wnd=w;	}
+	int Draw(mglGraph *gr)
+	{
+		gr->Line(mglPoint(),pnt,"Ar2");
+		char str[16];	snprintf(str,15,"i=%ld",i);
+		gr->Puts(mglPoint(),str);
+		return 0;
+	}
+	void Calc()
+	{
+		for(i=0;;i++)	// do calculation
+		{
+			Check();	// check if need pause
+			long_calculations();// which can be very long
+			pnt.Set(2*mgl_rnd()-1,2*mgl_rnd()-1);
+			if(wnd)	wnd->Update();
+		}
+	}
+} dr;
+#endif
+//-----------------------------------------------------------------------------
 int main(int argc,char **argv)
 {
 	mglFLTK *gr;
 	char key = 0;
 	if(argc>1)	key = argv[1][0]!='-' ? argv[1][0]:argv[1][1];
-	else	printf("You may specify argument '1', '2', '3' or 'd' for viewing examples of 1d, 2d, 3d or dual plotting\n");
+	else	printf("You may specify argument '1', '2', '3', 'd' for viewing examples of 1d, 2d, 3d, dual plotting,\nor 'm' for multi-threading sample.\n");
 	switch(key)
 	{
 	case '0':	gr = new mglFLTK((mglDraw *)NULL,"1D plots");	break;
@@ -100,6 +131,10 @@ int main(int argc,char **argv)
 	case '3':	gr = new mglFLTK(sample_3,"3D plots");	break;
 	case 'd':	gr = new mglFLTK(sample_d,"Dual plots");break;
 	case 't':	gr = new mglFLTK(test_wnd,"Testing");	break;
+#if MGL_HAVE_PTHR_WIDGET
+	case 'm':	gr = new mglFLTK(&dr,"Multi-threading test");
+	dr.SetWnd(gr);	dr.Run();	break;
+#endif
 	default:	gr = new mglFLTK(sample,"Drop and waves");	break;
 	}
 	if(key=='0')
