@@ -34,6 +34,9 @@ void MGL_EXPORT mgl_delete_graph_(uintptr_t *gr);
 /// Set size of frame in pixels. Normally this function is called internally.
 void MGL_EXPORT mgl_set_size(HMGL gr, int width, int height);
 void MGL_EXPORT mgl_set_size_(uintptr_t *gr, int *width, int *height);
+/// Set size of frame in pixels, but don't erase primitives
+void MGL_EXPORT mgl_scale_size(HMGL gr, int width, int height);
+void MGL_EXPORT mgl_scale_size_(uintptr_t *gr, int *width, int *height);
 /// Scaling for all further set size calls.
 void MGL_EXPORT mgl_set_size_scl(double scl);
 void MGL_EXPORT mgl_set_size_scl_(double *scl);
@@ -50,6 +53,10 @@ void MGL_EXPORT mgl_finish_(uintptr_t *gr);
 void MGL_EXPORT mgl_rasterize(HMGL gr);
 void MGL_EXPORT mgl_rasterize_(uintptr_t *gr);
 
+/// Set the size of semi-transparent area around lines, marks, glyphs, ... Default is 1.
+void MGL_EXPORT mgl_pen_delta(HMGL gr, double d);
+void MGL_EXPORT mgl_pen_delta_(uintptr_t *gr, double *d);
+
 /// Set tick length
 void MGL_EXPORT mgl_set_tick_len(HMGL gr, double len, double stt);
 void MGL_EXPORT mgl_set_tick_len_(uintptr_t *gr, mreal *len, mreal *stt);
@@ -504,11 +511,12 @@ void MGL_EXPORT mgl_zoom(HMGL gr, double x1, double y1, double x2, double y2);
 void MGL_EXPORT mgl_zoom_(uintptr_t *gr, mreal *x1, mreal *y1, mreal *x2, mreal *y2);
 
 //-----------------------------------------------------------------------------
-#if MGL_HAVE_PTHREAD
-void MGL_EXPORT mgl_draw_thr(void *);
-#endif
 /// Callback function for mouse click
 void MGL_EXPORT mgl_set_click_func(HMGL gr, void (*func)(void *p));
+#if MGL_HAVE_PTHR_WIDGET
+/// Mutex for lock/unlock by widget
+void MGL_EXPORT mgl_wnd_set_mutex(HMGL gr, pthread_mutex_t *mutex);
+#endif
 
 /// Set callback functions for drawing and data reloading
 void MGL_EXPORT mgl_wnd_set_func(HMGL gr, int (*draw)(HMGL gr, void *p), void *par, void (*reload)(void *p));
