@@ -22,13 +22,14 @@
 
 #include "mgl2/data.h"
 #include "mgl2/datac_cf.h"
-#ifdef __cplusplus
 //-----------------------------------------------------------------------------
 #include <vector>
 #include <string>
-#define mgl2 	mreal(2)
-#define mgl3 	mreal(3)
-#define mgl4 	mreal(4)
+//-----------------------------------------------------------------------------
+#ifndef SWIG
+dual MGL_EXPORT mglLinearC(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
+dual MGL_EXPORT mglSpline3C(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z,dual *dx=0, dual *dy=0, dual *dz=0);
+dual MGL_EXPORT mglSpline3Cs(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
 //-----------------------------------------------------------------------------
 /// Class for working with complex data array
 class MGL_EXPORT mglDataC : public mglDataA
@@ -71,6 +72,12 @@ using mglDataA::Momentum;
 	mglDataC(const double *d, int rows, int cols)	{	a=0;	Set(d,cols,rows);	}
 	mglDataC(const float *d, int size)	{	a=0;	Set(d,size);	}
 	mglDataC(const float *d, int rows, int cols)	{	a=0;	Set(d,cols,rows);	}
+	/// Allocate memory and copy data from std::vector<T>
+	mglDataC(const std::vector<int> &d)		{	a=0;	Set(d);	}
+	mglDataC(const std::vector<float> &d)	{	a=0;	Set(d);	}
+	mglDataC(const std::vector<double> &d)	{	a=0;	Set(d);	}
+	mglDataC(const std::vector<std::complex<double> > &d)	{	a=0;	Set(d);	}
+	mglDataC(const std::vector<std::complex<float> > &d)	{	a=0;	Set(d);	}
 	/// Read data from file
 	mglDataC(const char *fname)			{	a=0;	Read(fname);	}
 	/// Allocate the memory for data array and initialize it zero
@@ -127,17 +134,18 @@ using mglDataA::Momentum;
 	{	mgl_datac_set_ap(this, &ampl, &phase);	}
 	/// Allocate memory and copy data from std::vector<T>
 	inline void Set(const std::vector<int> &d)
-	{	if(d.size()<1)	return;
-		Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+	{	if(d.size()>0)	{	Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+		else	Create(1);	}
 	inline void Set(const std::vector<float> &d)
-	{	if(d.size()<1)	return;
-		Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+	{	if(d.size()>0)	Set(&(a[0]),d.size());	else	Create(1);	}
 	inline void Set(const std::vector<double> &d)
-	{	if(d.size()<1)	return;
-		Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
-	inline void Set(const std::vector<dual> &d)
-	{	if(d.size()<1)	return;
-		Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+	{	if(d.size()>0)	Set(&(a[0]),d.size());	else	Create(1);	}
+	inline void Set(const std::vector<std::complex<double> > &d)
+	{	if(d.size()>0)	{	Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+		else	Create(1);	}
+	inline void Set(const std::vector<std::complex<float> > &d)
+	{	if(d.size()>0)	{	Create(d.size());	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+		else	Create(1);	}
 
 	/// Create or recreate the array with specified size and fill it by zero
 	inline void Create(long mx,long my=1,long mz=1)
@@ -186,7 +194,7 @@ using mglDataA::Momentum;
 	/// Equidistantly fill the data to range [x1,x2] in direction dir
 	inline void Fill(dual x1,dual x2=mglNaN,char dir='x')
 	{	mgl_datac_fill(this,x1,x2,dir);	}
-	
+
 		/// Fill the data by interpolated values of vdat parametrically depended on xdat,ydat,zdat for x,y,z in range [p1,p2] using global spline
 	inline void RefillGS(const mglDataA &xdat, const mglDataA &vdat, mreal x1, mreal x2,long sl=-1)
 	{	mgl_datac_refill_gs(this,&xdat,&vdat,x1,x2,sl);	}
@@ -230,6 +238,10 @@ using mglDataA::Momentum;
 	/// Save whole data array (for ns=-1) or only ns-th slice to text file
 	void Save(const char *fname,long ns=-1) const
 	{	mgl_datac_save(this,fname,ns);	}
+	/// Get whole data array (for ns=-1) or only ns-th slice to string
+	std::string Get(long ns=-1) const
+	{	return mgl_datac_to_string(this,ns);	}
+
 	/// Read data from tab-separated text files with auto determining size which filenames are result of sprintf(fname,templ,t) where t=from:step:to
 	inline bool ReadRange(const char *templ, double from, double to, double step=1, bool as_slice=false)
 	{	return mgl_datac_read_range(this,templ,from,to,step,as_slice);	}
@@ -253,9 +265,12 @@ using mglDataA::Momentum;
 	/// Get imaginary part of data values
 	inline mglData Imag() const
 	{	return mglData(true,mgl_datac_imag(this));	}
-	/// Get absolute value of data values
+	/// Get absolute value of data values, i.e. |u|
 	inline mglData Abs() const
 	{	return mglData(true,mgl_datac_abs(this));	}
+	/// Get square of absolute value of data values, i.e. |u|^2
+	inline mglData Norm() const
+	{	return mglData(true,mgl_datac_norm(this));	}
 	/// Get argument of data values
 	inline mglData Arg() const
 	{	return mglData(true,mgl_datac_arg(this));	}
@@ -355,10 +370,10 @@ using mglDataA::Momentum;
 	{	return mgl_datac_spline(this, x*(nx-1),y*(ny-1),z*(nz-1));	}
 	/// Interpolate by linear function the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	inline dual Linear(mreal x,mreal y=0,mreal z=0)	const
-	{	return mgl_datac_linear(this,x,y,z);	}
+	{	return mgl_datac_linear_ext(this,x,y,z,0,0,0);	}
 	/// Interpolate by line the data to given point x,\a y,\a z which normalized in range [0, 1]
 	inline dual Linear1(mreal x,mreal y=0,mreal z=0) const
-	{	return mgl_datac_linear(this,x*(nx-1),y*(ny-1),z*(nz-1));	}
+	{	return mgl_datac_linear_ext(this,x*(nx-1),y*(ny-1),z*(nz-1),0,0,0);	}
 	/// Interpolate by linear function the data and return its derivatives at given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	inline dual Linear(mglPoint &dif, mreal x,mreal y=0,mreal z=0)	const
 	{
@@ -430,25 +445,29 @@ using mglDataA::Momentum;
 	/// Set the value in given cell of the data
 	void set_v(mreal val, long i,long j=0,long k=0)	{	mgl_datac_set_value(this,val,i,j,k);	}
 #endif
+	/// Get the interpolated value and its derivatives in given data cell without border checking
+	mreal valueD(mreal x,mreal y=0,mreal z=0,mreal *dx=0,mreal *dy=0,mreal *dz=0) const
+	{	dual aa,ax,ay,az;	mreal res;
+		aa = mglSpline3C(a,nx,ny,nz,x,y,z,&ax,&ay,&az);	res = abs(aa);
+		if(dx)	*dx = res?(real(aa)*real(ax)+imag(aa)*imag(ax))/res:0;
+		if(dy)	*dy = res?(real(aa)*real(ay)+imag(aa)*imag(ay))/res:0;
+		if(dz)	*dz = res?(real(aa)*real(az)+imag(aa)*imag(az))/res:0;	return res;	}
+	/// Get the interpolated value in given data cell without border checking
+	mreal value(mreal x,mreal y=0,mreal z=0) const
+	{	return abs(mglSpline3Cs(a,nx,ny,nz,x,y,z));	}
 	mreal vthr(long i) const {	return abs(a[i]);	}
 	// add for speeding up !!!
 	mreal dvx(long i,long j=0,long k=0) const
 	{   register long i0=i+nx*(j+ny*k);
-		return i>0? abs(i<nx-1? (a[i0+1]-a[i0-1])/mgl2:a[i0]-a[i0-1]) : abs(a[i0+1]-a[i0]);	}
+		return i>0? abs(i<nx-1? (a[i0+1]-a[i0-1])/mreal(2):a[i0]-a[i0-1]) : abs(a[i0+1]-a[i0]);	}
 	mreal dvy(long i,long j=0,long k=0) const
 	{   register long i0=i+nx*(j+ny*k);
-		return j>0? abs(j<ny-1? (a[i0+nx]-a[i0-nx])/mgl2:a[i0]-a[i0-nx]) : abs(a[i0+nx]-a[i0]);}
+		return j>0? abs(j<ny-1? (a[i0+nx]-a[i0-nx])/mreal(2):a[i0]-a[i0-nx]) : abs(a[i0+nx]-a[i0]);}
 	mreal dvz(long i,long j=0,long k=0) const
 	{   register long i0=i+nx*(j+ny*k), n=nx*ny;
-		return k>0? abs(k<nz-1? (a[i0+n]-a[i0-n])/mgl2:a[i0]-a[i0-n]) : abs(a[i0+n]-a[i0]);	}
+		return k>0? abs(k<nz-1? (a[i0+n]-a[i0-n])/mreal(2):a[i0]-a[i0-n]) : abs(a[i0+n]-a[i0]);	}
 };
 //-----------------------------------------------------------------------------
-#ifndef SWIG
-dual MGL_EXPORT mglLinearC(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
-dual MGL_EXPORT mglSpline3C(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z,dual *dx=0, dual *dy=0, dual *dz=0);
-dual MGL_EXPORT mglSpline3Cs(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
-#endif
-//-----------------------------------------------------------------------------
 /// Saves result of PDE solving (|u|^2) for "Hamiltonian" ham with initial conditions ini
 inline mglDataC mglPDEc(mglBase *gr, const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, mreal dz=0.1, mreal k0=100,const char *opt="")
 {	return mglDataC(true, mgl_pde_solve_c(gr,ham, &ini_re, &ini_im, dz, k0,opt));	}
