@@ -1,5 +1,5 @@
 /***************************************************************************
- * window.h is part of Math Graphic Library
+ * qmathgl.h is part of Math Graphic Library
  * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -44,6 +44,7 @@ public:
 	bool enableMouse;	///< Enable mouse handlers
 	bool enableWheel;	///< Enable mouse wheel handlers
 	QString primitives;	///< Manual primitives, defined by user
+	mglCanvas *gr;		///< Built-in mglCanvas instance (mglCanvasQT is used by default)
 
 	QMathGL(QWidget *parent = 0, Qt::WindowFlags f = 0);
 	virtual ~QMathGL();
@@ -61,6 +62,11 @@ public:
 	{	setDraw(func?mgl_draw_graph:0,(void*)func);	}
 	inline void zoomRegion(mreal xx1,mreal xx2,mreal yy1, mreal yy2)
 	{	x1=xx1;	y1=yy1;	x2=xx2;	y2=yy2;	}
+	/// Get mglDraw pointer or NULL
+	inline mglDraw *getClass()
+	{	mglDraw *d=0;
+		if(draw_func==mgl_draw_class)	d = (mglDraw*)draw_par;
+		if(draw)	d = draw;	return d;	}
 
 	int getPer() const	{return int(per);}	///< Get perspective value
 	int getPhi() const	{return int(phi);}	///< Get Phi-angle value
@@ -70,6 +76,7 @@ public:
 	bool getZoom() const	{return zoom;}	///< Get mouse zooming state
 	bool getRotate() const	{return rotate;}///< Get mouse rotation state
 	bool getViewYZ() const	{return viewYZ;}///< Get mouse rotation axis
+	bool getPause() const	{return pause;}	///< Get calculation pause state
 	bool isActive(int xs,int ys);	///< Check if active point is pressed
 
 public slots:
@@ -94,6 +101,7 @@ public slots:
 	void setCustDraw(bool a);	///< Switch on/off using custom draw
 	void setZoom(bool z);	///< Switch on/off mouse zooming
 	void setRotate(bool r);	///< Switch on/off mouse rotation
+	void setPause(bool p);	///< Switch on/off calculation pause
 	void zoomIn();			///< Zoom in graphics
 	void zoomOut();			///< Zoom out graphics
 	void restore();			///< Restore zoom and rotation to default values
@@ -152,6 +160,7 @@ signals:
 	void lightChanged(bool);	///< Lighting changed (by toolbar)
 	void zoomChanged(bool);		///< Zooming changed (by toolbar)
 	void rotateChanged(bool);	///< Rotation changed (by toolbar)
+	void pauseChanged(bool);	///< Pause changed (by toolbar)
 	void usePrimChanged(bool);	///< Use primitive changed (i.e. have or not drawing function)
 	void viewYZChanged(bool);	///< Rotation axis changed (by toolbar)
 	void mouseClick(mreal,mreal,mreal);	///< Position of mouse click
@@ -176,7 +185,6 @@ protected:
 	void wheelEvent(QWheelEvent *);
 	void mouseDoubleClickEvent(QMouseEvent *);
 
-	mglCanvas *gr;		///< Built-in mglCanvasQT-er instance (used by default)
 	void *draw_par;		///< Parameters for drawing function mglCanvasWnd::DrawFunc.
 	/// Drawing function for window procedure. It should return the number of frames.
 	int (*draw_func)(mglBase *gr, void *par);
@@ -188,6 +196,7 @@ protected:
 	double per;			///< Value of perspective ( must be in [0,1) )
 	bool alpha;			///< Transparency state
 	bool light;			///< Lightning state
+	bool pause;			///< Pause state
 	bool custZoom;		///< Use custom zoom instead of built in
 	bool custDraw;		///< Use custom draw before main drawing
 	bool zoom;			///< Mouse zoom state
