@@ -22,11 +22,13 @@
 
 #include "mgl2/qt.h"
 #include "mgl2/fltk.h"
-#include "mgl2/wx.h"
+//#include "mgl2/wx.h"
 //-----------------------------------------------------------------------------
 /// Wrapper class for all windows displaying graphics
 class MGL_EXPORT mglWindow : public mglWnd
 {
+	mglWindow(const mglWindow &t) {}	// copying is not allowed
+	const mglWindow &operator=(const mglWindow &t)	{	return t;	}
 	int wnd;	///< Type of window
 public:
 	mglWindow(const char *title="MathGL") : mglWnd()
