@@ -1,7 +1,7 @@
 #pragma once
 
-#include <QtCore/QStringList>
-#include <QtCore/QObject>
+#include <QObject>
+#include <QStringList>
 
 class Backend : public QObject
 {
