@@ -125,7 +125,7 @@ mglBase::mglBase()
 
 	strcpy(last_style,"__1 {dFFFF}k\0");
 	MinS.Set(-1,-1,-1);	MaxS.Set(1,1,1);
-	fnt = new mglFont;	fnt->gr = this;	PrevState=NAN;
+	fnt = new mglFont;	fnt->gr = this;	PrevState=NAN;	size_opt=NAN;
 }
 mglBase::~mglBase()
 {
@@ -187,11 +187,11 @@ const char *mglWarn[mglWarnEnd] = {"data dimension(s) is incompatible",	//mglWar
 								"axis ranges are incompatible",			//mglWarnTern
 								"pointer is NULL",						//mglWarnNull
 								"not enough space for plot",			//mglWarnSpc
-								"There are wrong argument(s) in script",//mglScrArg
-								"There are wrong command in script",	//mglScrCmd
-								"There are too long string in script",	//mglScrLong
-								"There are unbalanced ' in script",		//mglScrStr
-								"There are changing temporary data in script"};	//mglScrTemp
+								"There is wrong argument(s) in script",	//mglScrArg
+								"There is wrong command(s) in script",	//mglScrCmd
+								"There is too long string(s) in script",	//mglScrLong
+								"There is unbalanced ' in script",		//mglScrStr
+								"There is changing temporary data in script"};	//mglScrTemp
 //-----------------------------------------------------------------------------
 extern bool mglPrintWarn;
 void mglBase::SetWarn(int code, const char *who)
@@ -251,6 +251,17 @@ long mglBase::AddGlyph(int s, long j)
 //-----------------------------------------------------------------------------
 //		Add points to the buffer
 //-----------------------------------------------------------------------------
+void inline mgl_put_inbox(mreal a1, mreal a2, mreal &a)
+{
+	if(a1<a2)	{	if(a<a1)	a=a1;	if(a>a2)	a=a2;	}
+	else		{	if(a<a2)	a=a2;	if(a>a1)	a=a1;	}
+}
+void MGL_NO_EXPORT mgl_coor_box(HMGL gr, mglPoint &p)
+{
+	mgl_put_inbox(gr->Min.x, gr->Max.x, p.x);
+	mgl_put_inbox(gr->Min.y, gr->Max.y, p.y);
+	mgl_put_inbox(gr->Min.z, gr->Max.z, p.z);
+}
 long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mreal a, int scl)
 {
 	// scl=0 -- no scaling
@@ -258,9 +269,14 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 	// scl&2 -- disable NAN at scaling
 	// scl&4 -- disable NAN for normales if no light
 	// scl&8 -- bypass palette for enabling alpha
+	// scl&16 -- put points inside axis range
 	if(mgl_isnan(c) || mgl_isnan(a))	return -1;
 	bool norefr = mgl_isnan(n.x) && mgl_isnan(n.y) && !mgl_isnan(n.z);
-	if(scl>0)	ScalePoint(mat,p,n,!(scl&2));
+	if(scl>0)
+	{
+		if(scl&16)	mgl_coor_box(this, p);
+		ScalePoint(mat,p,n,!(scl&2));
+	}
 	if(mgl_isnan(p.x))	return -1;
 	a = (a>=0 && a<=1) ? a : AlphaDef;
 	c = (c>=0) ? c:CDef;
@@ -442,12 +458,36 @@ bool mglBase::ScalePoint(const mglMatrix *, mglPoint &p, mglPoint &n, bool use_n
 	}
 	else
 	{
-		if(x1<Min.x)	{x=Min.x;	n.Set(1,0,0);}
-		if(x2>Max.x)	{x=Max.x;	n.Set(1,0,0);}
-		if(y1<Min.y)	{y=Min.y;	n.Set(0,1,0);}
-		if(y2>Max.y)	{y=Max.y;	n.Set(0,1,0);}
-		if(z1<Min.z)	{z=Min.z;	n.Set(0,0,1);}
-		if(z2>Max.z)	{z=Max.z;	n.Set(0,0,1);}
+		if(Min.x<Max.x)
+		{
+			if(x1<Min.x)	{x=Min.x;	n.Set(1,0,0);}
+			if(x2>Max.x)	{x=Max.x;	n.Set(1,0,0);}
+		}
+		else
+		{
+			if(x1<Max.x)	{x=Max.x;	n.Set(1,0,0);}
+			if(x2>Min.x)	{x=Min.x;	n.Set(1,0,0);}
+		}
+		if(Min.y<Max.y)
+		{
+			if(y1<Min.y)	{y=Min.y;	n.Set(0,1,0);}
+			if(y2>Max.y)	{y=Max.y;	n.Set(0,1,0);}
+		}
+		else
+		{
+			if(y1<Max.y)	{y=Max.y;	n.Set(0,1,0);}
+			if(y2>Min.y)	{y=Min.y;	n.Set(0,1,0);}
+		}
+		if(Min.z<Max.z)
+		{
+			if(z1<Min.z)	{z=Min.z;	n.Set(0,0,1);}
+			if(z2>Max.z)	{z=Max.z;	n.Set(0,0,1);}
+		}
+		else
+		{
+			if(z1<Max.z)	{z=Max.z;	n.Set(0,0,1);}
+			if(z2>Min.z)	{z=Min.z;	n.Set(0,0,1);}
+		}
 	}
 
 	x1=x;	y1=y;	z1=z;
@@ -494,7 +534,7 @@ bool mglBase::ScalePoint(const mglMatrix *, mglPoint &p, mglPoint &n, bool use_n
 //-----------------------------------------------------------------------------
 void mglScaleAxis(mreal &v1, mreal &v2, mreal &v0, mreal x1, mreal x2)
 {
-	if(x1==x2 || v1==v2)	return;
+	if(!mgl_isrange(x1,x2) || !mgl_isrange(v1,v2))	return;
 	mreal dv,d0;	x2-=1;
 	if(v1*v2>0 && (v2/v1>=100 || v2/v1<=0.01))	// log scale
 	{
@@ -523,11 +563,11 @@ void mglBase::SetOrigin(mreal x0, mreal y0, mreal z0, mreal c0)
 //-----------------------------------------------------------------------------
 void mglBase::SetRanges(mglPoint m1, mglPoint m2)
 {
-	if(m1.x!=m2.x)	{	Min.x=m1.x;	Max.x=m2.x;	}
-	if(m1.y!=m2.y)	{	Min.y=m1.y;	Max.y=m2.y;	}
-	if(m1.z!=m2.z)	{	Min.z=m1.z;	Max.z=m2.z;	}
-	if(m1.c!=m2.c)	{	Min.c=m1.c;	Max.c=m2.c;	}
-	else			{	Min.c=Min.z;Max.c=Max.z;}
+	if(mgl_isrange(m1.x, m2.x))	{	Min.x=m1.x;	Max.x=m2.x;	}
+	if(mgl_isrange(m1.y, m2.y))	{	Min.y=m1.y;	Max.y=m2.y;	}
+	if(mgl_isrange(m1.z, m2.z))	{	Min.z=m1.z;	Max.z=m2.z;	}
+	if(mgl_isrange(m1.c, m2.c))	{	Min.c=m1.c;	Max.c=m2.c;	}
+	else	{	Min.c=Min.z;Max.c=Max.z;}
 
 	if(Org.x<Min.x && mgl_isnum(Org.x))	Org.x = Min.x;
 	if(Org.x>Max.x && mgl_isnum(Org.x))	Org.x = Max.x;
@@ -557,7 +597,7 @@ void mglBase::CRange(HCDT a,bool add, mreal fact)
 }
 void mglBase::CRange(mreal v1,mreal v2,bool add)
 {
-	if(v1==v2 && !add)	return;
+	if(!mgl_isrange(v1,v2) && !add)	return;
 	if(!add)
 	{
 		if(mgl_isnum(v1))	Min.c = v1;
@@ -592,7 +632,7 @@ void mglBase::XRange(HCDT a,bool add,mreal fact)
 }
 void mglBase::XRange(mreal v1,mreal v2,bool add)
 {
-	if(v1==v2 && !add)	return;
+	if(!mgl_isrange(v1,v2) && !add)	return;
 	if(!add)
 	{
 		if(mgl_isnum(v1))	Min.x = v1;
@@ -627,7 +667,7 @@ void mglBase::YRange(HCDT a,bool add,mreal fact)
 }
 void mglBase::YRange(mreal v1,mreal v2,bool add)
 {
-	if(v1==v2 && !add)	return;
+	if(!mgl_isrange(v1,v2) && !add)	return;
 	if(!add)
 	{
 		if(mgl_isnum(v1))	Min.y = v1;
@@ -663,7 +703,7 @@ void mglBase::ZRange(HCDT a,bool add,mreal fact)
 }
 void mglBase::ZRange(mreal v1,mreal v2,bool add)
 {
-	if(v1==v2 && !add)	return;
+	if(!mgl_isrange(v1,v2) && !add)	return;
 	if(!add)
 	{
 		if(mgl_isnum(v1))	Min.z = v1;
@@ -692,10 +732,10 @@ void mglBase::ZRange(mreal v1,mreal v2,bool add)
 //-----------------------------------------------------------------------------
 void mglBase::SetAutoRanges(mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, mreal z2, mreal c1, mreal c2)
 {
-	if(x1!=x2)	{	Min.x = x1;	Max.x = x2;	}
-	if(y1!=y2)	{	Min.y = y1;	Max.y = y2;	}
-	if(z1!=z2)	{	Min.z = z1;	Max.z = z2;	}
-	if(c1!=c2)	{	Min.c = c1;	Max.c = c2;	}
+	if(mgl_isrange(x1,x2))	{	Min.x = x1;	Max.x = x2;	}
+	if(mgl_isrange(y1,y2))	{	Min.y = y1;	Max.y = y2;	}
+	if(mgl_isrange(z1,z2))	{	Min.z = z1;	Max.z = z2;	}
+	if(mgl_isrange(c1,c2))	{	Min.c = c1;	Max.c = c2;	}
 }
 //-----------------------------------------------------------------------------
 void mglBase::Ternary(int t)
@@ -1237,6 +1277,7 @@ mreal mglBase::SaveState(const char *opt)
 		mgl_strtrim(b);
 
 		mreal ff=atof(b),ss;
+		size_opt = NAN;
 		if(!strcmp(b,"on"))	ff=1;
 		if(!strcmp(a+1,"range"))
 		{
@@ -1255,7 +1296,7 @@ mreal mglBase::SaveState(const char *opt)
 		else if(!strcmp(a,"ambient"))	SetAmbient(ff);
 		else if(!strcmp(a,"diffuse"))	SetDifLight(ff);
 		else if(!strcmp(a,"size"))
-		{	SetMarkSize(ff);	SetFontSize(ff);	SetArrowSize(ff);	}
+		{	SetMarkSize(ff);	SetFontSize(ff);	SetArrowSize(ff);	size_opt=ff;	}
 		else if(!strcmp(a,"num") || !strcmp(a,"number") || !strcmp(a,"value"))	res=ff;
 		else if(!strcmp(a,"legend"))
 		{	if(*b=='\'')	{	b++;	b[strlen(b)-1]=0;	}	leg_str = b;	}
