@@ -223,7 +223,7 @@ int Height;			///< Height of the image
 int Depth;			///< Depth of the image
 int CurFrameId;		///< Number of automaticle created frames
 GifFileType *gif;*/
-	SetDrawReg(1,1,0);		Perspective(0);
+	SetDrawReg(1,1,0);		Perspective(0);	SetPenDelta(1);
 	memcpy(mgl_mask_val, mgl_mask_def, 16*sizeof(uint64_t));	// should be > 16*8
 	ax.Clear();	ay.Clear();	az.Clear();	ac.Clear();
 	mgl_clear_fft();		DefMaskAn=0;	ResetMask();
@@ -497,8 +497,7 @@ pthread_mutex_lock(&mutexPtx);
 			mglColor mc = Txt[long(col1)].GetC(col1);
 			mglPrim a(6);	a.n1 = p;
 			a.n2 = int(255*mc.r) + 256*(int(255*mc.g) + 256*int(255*mc.b));
-			mglText txt(text,font);
-			a.n3 = Ptx.size();	Ptx.push_back(txt);
+			a.n3 = Ptx.size();	Ptx.push_back(mglText(text,font));
 			a.s = size;	a.w = shift;	a.p=ftet;
 			add_prim(a);
 		}
@@ -530,6 +529,7 @@ pthread_mutex_lock(&mutexPtx);
 			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
 #pragma omp critical(pnt)
 			{k4=Pnt.size();	MGL_PUSH(Pnt,pt,mutexPnt);}
+			PDef = 0xffff;	// reset to solid line
 			line_plot(k1,k2);	line_plot(k1,k3);
 			line_plot(k4,k2);	line_plot(k4,k3);
 			mreal bl = AddTexture('w');
@@ -831,15 +831,18 @@ std::wstring MGL_EXPORT mgl_ftoa(double v, const char *fmt)
 
 	// clear exp format
 	int st = se[0]=='-'?1:0;
-	if(plus || se[3+st+dig]=='-')	// first remove zeros after 'e'
+	if(strcmp(sf,"nan"))
 	{
-		for(i=(dig>0?4:3)+st+dig;i<le && se[i]=='0';i++);
-		memmove(se+(dig>0?4:3)+st+dig,se+i,le-i+1);
-	}
-	else
-	{
-		for(i=(dig>0?3:2)+st+dig;i<le && (se[i]=='0' || se[i]=='+');i++);
-		memmove(se+(dig>0?3:2)+st+dig,se+i,le-i+1);
+		if(plus || se[3+st+dig]=='-')	// first remove zeros after 'e'
+		{
+			for(i=(dig>0?4:3)+st+dig;i<le && se[i]=='0';i++);
+			memmove(se+(dig>0?4:3)+st+dig,se+i,le-i+1);
+		}
+		else
+		{
+			for(i=(dig>0?3:2)+st+dig;i<le && (se[i]=='0' || se[i]=='+');i++);
+			memmove(se+(dig>0?3:2)+st+dig,se+i,le-i+1);
+		}
 	}
 	le=strlen(se);
 	// don't allow '+' at the end
@@ -909,11 +912,11 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 		w = w>t ? w:t;
 	}
 	w += ll+0.01*iw;	// add space for lines
-	j = long((ih*0.95)/h);
+	j = long((ih*0.95)/h);	if(j<1)	j=1;
 	long ncol = 1+(n-1)/j, nrow = (n+ncol-1)/ncol;
 	if(strchr(font,'-'))	// horizontal legend
 	{
-		j = long((iw*0.95)/w);
+		j = long((iw*0.95)/w);	if(j<1)	j=1;
 		nrow = 1+(n-1)/j;
 		ncol = (n+nrow-1)/nrow;
 	}
