@@ -280,6 +280,12 @@ void MGL_EXPORT mgl_set_size(HMGL gr, int width, int height)
 	width = int(mgl_size_scl*width);	height = int(mgl_size_scl*height);
 	if(g)	g->SetSize(width, height);
 }
+void MGL_EXPORT mgl_scale_size(HMGL gr, int width, int height)
+{
+	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);
+	width = int(mgl_size_scl*width);	height = int(mgl_size_scl*height);
+	if(g)	g->SetSize(width, height, false);
+}
 void MGL_EXPORT mgl_set_def_param(HMGL gr)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->DefaultPlotParam();	}
 void MGL_EXPORT mgl_combine_gr(HMGL gr, HMGL in)
@@ -362,6 +368,8 @@ void MGL_EXPORT mgl_delete_graph_(uintptr_t *gr)	{	delete _GR_;	}
 void MGL_EXPORT mgl_set_size_scl_(double *scl)	{	mgl_set_size_scl(*scl);	}
 void MGL_EXPORT mgl_set_size_(uintptr_t *gr, int *width, int *height)
 {	mgl_set_size(_GR_,*width,*height);	}
+void MGL_EXPORT mgl_scale_size_(uintptr_t *gr, int *width, int *height)
+{	mgl_scale_size(_GR_,*width,*height);	}
 void MGL_EXPORT mgl_set_def_param_(uintptr_t *gr)	{	_GR_->DefaultPlotParam();	}
 void MGL_EXPORT mgl_combine_gr_(uintptr_t *gr, uintptr_t *in)
 {	_GR_->Combine((mglCanvas *)in);	}
@@ -483,3 +491,7 @@ void MGL_EXPORT mgl_rasterize(HMGL gr)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->Rasterize();	}
 void MGL_EXPORT mgl_rasterize_(uintptr_t *gr)	{	_GR_->Rasterize();	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_pen_delta(HMGL gr, double d)
+{	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->SetPenDelta(d);	}
+void MGL_EXPORT mgl_pen_delta_(uintptr_t *gr, double *d)	{	_GR_->SetPenDelta(*d);	}
+//-----------------------------------------------------------------------------
