@@ -20,6 +20,7 @@
 #include <float.h>
 #include <math.h>
 #include <list>
+#include <limits>
 #include "mgl2/other.h"
 #include "mgl2/data.h"
 #include "mgl2/thread.h"
@@ -44,7 +45,7 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 	{
 		mglPoint p1,p2,p3,q;
 		gr->Reserve(m*3);
-		for(long i=0;i<m;i++)	if(nums->v(0,i)>=0 && nums->v(1,i)>=0 && nums->v(2,i)>=0) 
+		for(long i=0;i<m;i++)	if(nums->v(0,i)>=0 && nums->v(1,i)>=0 && nums->v(2,i)>=0)
 		{
 			register long k1 = long(nums->v(0,i)+0.5);
 			p1.Set(x->v(k1), y->v(k1), z->v(k1));
@@ -497,30 +498,65 @@ HMDT MGL_EXPORT mgl_triangulation_2d(HCDT x, HCDT y)
 	if(y->GetNN()!=n)	return nums;
 	// use s-hull here
 	std::vector<Shx> pts;
-	std::vector<long> out;
 	Shx pt;
 
-	double mx = 0, my = 0;
+	double x1=mglInf, x2=-mglInf, y1=mglInf, y2=-mglInf;
 	for(long i=0;i<n;i++)
 	{
-		register double t;
-		t = fabs(x->vthr(i));	if(t>mx)	mx=t;
-		t = fabs(y->vthr(i));	if(t>my)	my=t;
+		register mreal xx=x->vthr(i), yy = y->vthr(i);
+		if(xx<x1)	x1=xx;	if(xx>x2)	x2=xx;
+		if(yy<y1)	y1=yy;	if(yy>y2)	y2=yy;
 	}
-	mx *= 1e-15;	my *= 1e-15;
-	for(long i=0;i<n;i++)
+	const double dx=x2-x1, dy=y2-y1;
+	if(dx==0 || dy==0)	return nums;
+	for(long i=0;i<n;i++)	// Filter NaNs and Infs
 	{
-		pt.r = x->vthr(i);	pt.c = y->vthr(i);
+		pt.r = (x->vthr(i)-x1)/dx;	pt.c = (y->vthr(i)-y1)/dy;
 		if(mgl_isbad(pt.r) || mgl_isbad(pt.c))	continue;
-		if(fabs(pt.r)<mx)	pt.r=0;
-		if(fabs(pt.c)<my)	pt.c=0;
 		pt.id = i;    pts.push_back(pt);
 	}
 
 	std::vector<Triad> triads;
-	if(de_duplicate(pts, out))
-		mgl_set_global_warn("There are duplicated points for triangulation.");
-	s_hull_pro(pts, triads);
+	static const double float_eps = std::numeric_limits<float>::epsilon();
+	Dupex grid_step(float_eps, float_eps);
+	const size_t original_size = pts.size();
+
+	if(pts.size() >= 3u && 0. < grid_step.r && 0. < grid_step.c) {
+		std::vector<long> out;
+		de_duplicate(pts, out, grid_step);
+
+		if (pts.size() >= 3u && s_hull_pro(pts, triads) < 0) {
+			// Error occured. It may be caused by degenerated dataset. Well, let's try to increment rounding grid step.
+			// Why 4? Why not. There are no particular reasons for this.
+			grid_step.r *= 4.;
+			grid_step.c *= 4.;
+
+			out.clear();
+			triads.clear();
+
+			de_duplicate(pts, out, grid_step);
+
+			if (pts.size() >= 3u && s_hull_pro(pts, triads) < 0) {
+				// Last try. Let's assume uniform points distribution and use range / sqrt(pts.size()) * 2 as epsilon.
+				// It removes a 3/4 of points in optimal case but in the worst case it merges all points to the one.
+				const double density = 1. + floor(0.5 + std::sqrt(static_cast<double>(pts.size())));
+				grid_step.r = grid_step.c = 2/density;
+
+				out.clear();
+				de_duplicate(pts, out, grid_step);
+
+				triads.clear();
+				s_hull_pro(pts, triads);
+			}
+		}
+	}
+
+	if (triads.empty()) {
+		mgl_set_global_warn("Cannot triangulate this set!");
+	} else if(original_size > pts.size()) {
+		mgl_set_global_warn("There are duplicated or indistinguishably adjacent points for triangulation.");
+	}
+
 	long m = triads.size();
 	nums=new mglData(3,m);
 	for(long i=0;i<m;i++)
@@ -682,7 +718,7 @@ long MGL_NO_EXPORT mgl_get_next(long k1,long n,long *,long *set,mglPoint *qq)
 	for(i=0;i<n;i++)
 	{
 		if(i==k1 || set[i]>0)	continue;
-		r = mgl_norm(qq[i]-qq[k1]);
+		r = mgl_anorm(qq[i]-qq[k1]);
 		if(r<rm)	{	rm=r;	j=i;	}
 	}
 	return j;
@@ -698,7 +734,7 @@ long MGL_NO_EXPORT mgl_crust(long n,mglPoint *pp,long **nn,mreal ff)
 		for(rm = FLT_MAX,j=0;j<n;j++)
 		{
 			if(i==j)	continue;
-			r = mgl_norm(pp[i]-pp[j]);
+			r = mgl_anorm(pp[i]-pp[j]);
 			if(rm>r)	rm = r;
 		}
 		rs += sqrt(rm);
@@ -712,7 +748,7 @@ long MGL_NO_EXPORT mgl_crust(long n,mglPoint *pp,long **nn,mreal ff)
 		memset(set,0,100*sizeof(long));
 		for(ii=0,j=0;j<n;j++)	// find close vertexes
 		{
-			r = mgl_norm(pp[i]-pp[j]);
+			r = mgl_anorm(pp[i]-pp[j]);
 			if(r<=rs && j!=i)	{	ind[ii] = j;	ii++;	if(ii==99)	break;}
 		}
 		if(ii<3)	continue;	// nothing to do
@@ -734,3 +770,69 @@ long MGL_NO_EXPORT mgl_crust(long n,mglPoint *pp,long **nn,mreal ff)
 	return m;
 }
 //-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_ifs_2d_point(HCDT A, mreal& x, mreal& y, mreal amax)
+{
+	long i, n=A->GetNy();
+	mreal r = amax*mgl_rnd(), sum_prob = 0, x1;
+	for(i=0;i<n;i++)
+	{
+		sum_prob += A->v(6,i);
+		if(r<sum_prob)	break;
+	}
+	x1= A->v(0,i)*x + A->v(1,i)*y + A->v(4,i);
+	y = A->v(2,i)*x + A->v(3,i)*y + A->v(5,i);	x = x1;
+}
+HMDT MGL_EXPORT mgl_data_ifs_2d(HCDT A, long n, long skip)
+{
+	if(!A || A->GetNx()<7 || n<1)	return 0;	// incompatible dimensions
+	mreal amax=0;
+	for(long i=0; i<A->GetNy(); i++)	amax += A->v(6,i);
+	if(amax<=0)	return 0;
+
+	mglData *f = new mglData(2,n);
+	mreal x = 0, y = 0;
+	for(long i=0; i<skip; i++)	mgl_ifs_2d_point(A, x, y,amax);
+	for(long i=0; i<n; i++)
+	{
+		mgl_ifs_2d_point(A, x, y, amax);
+		f->a[2*i] = x;	f->a[2*i+1] = y;
+	}
+	return f;
+}
+uintptr_t MGL_EXPORT mgl_data_ifs_2d_(uintptr_t *d, long *n, long *skip)
+{	return uintptr_t(mgl_data_ifs_2d(_DT_,*n,*skip));	}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_ifs_3d_point(HCDT A, mreal& x, mreal& y, mreal& z, mreal amax)
+{
+	int i, n=A->GetNy();
+	mreal r = amax*mgl_rnd(), sum_prob = 0, x1, y1;
+	for (i=0; i<n; i++)
+	{
+		sum_prob += A->v(12,i);
+		if(r < sum_prob)  break;
+	}
+	x1= A->v(0,i)*x + A->v(1,i)*y + A->v(2,i)*z + A->v(9,i);
+	y1= A->v(3,i)*x + A->v(4,i)*y + A->v(5,i)*z + A->v(10,i);
+	z = A->v(6,i)*x + A->v(7,i)*y + A->v(8,i)*z + A->v(11,i);
+	x = x1;	y = y1;
+}
+HMDT MGL_EXPORT mgl_data_ifs_3d(HCDT A, long n, long skip)
+{
+	if(!A || A->GetNx()<13 || n<1)	return 0;   // incompatible dimensions
+	mreal amax = 0;
+	for(int i=0; i<A->GetNy(); i++)	amax += A->v(12,i);
+	if(amax <= 0) return 0;
+
+	mglData *f = new mglData(3,n);
+	mreal x = 0, y = 0, z = 0;
+	for(long i=0; i<skip; i++)	mgl_ifs_3d_point(A, x, y, z, amax);
+	for(long i=0; i<n; i++)
+	{
+		mgl_ifs_3d_point(A, x, y, z, amax);
+		f->a[3*i] = x;	f->a[3*i+1] = y;	f->a[3*i+2] = z;
+	}
+	return f;
+}
+uintptr_t MGL_EXPORT mgl_data_ifs_3d_(uintptr_t *d, long *n, long *skip)
+{   return uintptr_t(mgl_data_ifs_3d(_DT_,*n,*skip));   }
+//-----------------------------------------------------------------------------
