@@ -22,13 +22,6 @@
 #include "mgl2/canvas_cf.h"
 #include "mgl2/base.h"
 //-----------------------------------------------------------------------------
-#ifdef WIN32
-#include <io.h>
-wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim)	{	return wcstok(wcs,delim);	}
-#define wcstok(a,b,c) wcstokw32(a,b)
-#else
-#include <unistd.h>
-#endif
 MGL_EXPORT void (*mgl_ask_func)(const wchar_t *, wchar_t *)=0;
 void MGL_EXPORT mgl_ask_gets(const wchar_t *quest, wchar_t *res)
 {	printf("%ls\n",quest);	if(!fgetws(res,1024,stdin))	*res=0;	}
@@ -59,6 +52,7 @@ void mglParser::ScanFunc(const wchar_t *line)
 	if(!line)
 	{	func.clear();	num=0;	return;	}
 	num++;
+	while(*line<=' ' && *line!=0)	line++;
 	if(wcsncmp(line,L"func",4) || line[4]>' ')	return;
 	register long i;
 	for(i=4;line[i]<=' ' || line[i]=='\'';i++);
@@ -167,6 +161,9 @@ mglParser::mglParser(bool setsize)
 mglParser::~mglParser()
 {
 	DeleteAll();	delete []fval;
+	for(size_t i=0;i<NumList.size();i++)	// force delete built-in variables
+		if(NumList[i])	delete NumList[i];
+	NumList.clear();
 #if MGL_HAVE_LTDL
 	lt_dlexit();
 #endif
@@ -897,7 +894,6 @@ void mglParser::Execute(mglGraph *gr, FILE *fp, bool print)
 	std::wstring str;
 	wchar_t ch;
 	while(!feof(fp) && size_t(ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
-//	while(!feof(fp))	str.push_back(fgetwc(fp));
 	Execute(gr,str.c_str());
 	if(print)	printf("%s\n",gr->Message());
 }
