@@ -26,6 +26,7 @@
 #include <QMessageBox>
 #include <QTextBrowser>
 #include <QToolButton>
+#include <QFileDialog>
 #include <mgl2/mgl.h>
 
 #include "newcmd_dlg.h"
@@ -37,7 +38,7 @@ extern QString pathHelp;
 //-----------------------------------------------------------------------------
 NewCmdDialog::NewCmdDialog(QWidget *p) : QDialog(p,Qt::WindowStaysOnTopHint)
 {
-	fillList();
+	replace = false;
 	QPushButton *b;
 	QToolButton *t;
 	QHBoxLayout *m = new QHBoxLayout(this), *a;
@@ -47,9 +48,10 @@ NewCmdDialog::NewCmdDialog(QWidget *p) : QDialog(p,Qt::WindowStaysOnTopHint)
 	datDialog = new DataDialog(this);
 	type = new QComboBox(this);		o->addWidget(type);
 	type->setToolTip(tr("Groups of MGL commands"));
+	fillList();
 	name = new QComboBox(this);		o->addWidget(name);
 	name->setToolTip(tr("MGL commands for selected group"));
-	type->addItems(types);	name->addItems(argn[0]);
+	name->addItems(argn[0]);
 	info = new QLabel(this);		o->addWidget(info);
 	info->setToolTip(tr("Short description of selected command"));
 	kind= new QComboBox(this);		o->addWidget(kind);	kind->addItem("");
@@ -67,16 +69,11 @@ NewCmdDialog::NewCmdDialog(QWidget *p) : QDialog(p,Qt::WindowStaysOnTopHint)
 	connect(args,SIGNAL(cellDoubleClicked(int,int)), this, SLOT(insertData()));
 
 	a = new QHBoxLayout;	o->addLayout(a);
-	b = new QPushButton(tr("Add style"),this);	a->addWidget(b);
-	b->setToolTip(tr("Here you can select the plot style.\nThe result will be placed in 'fmt' argument."));
-	connect(b, SIGNAL(clicked()),this,SLOT(insertStl()));
-	b = new QPushButton(tr("Add data"),this);	a->addWidget(b);
-	connect(b, SIGNAL(clicked()),this,SLOT(insertData()));
 	b = new QPushButton(tr("Options"),this);	a->addWidget(b);
 	b->setToolTip(tr("Here you can specify command options.\nOptions are used for additional plot tunning."));
 	connect(b, SIGNAL(clicked()),this,SLOT(insertOpt()));
-	QLabel *l=new QLabel(tr("Options"));	o->addWidget(l);
-	opt = new QLineEdit(this);	o->addWidget(opt);
+	opt = new QLineEdit(this);	a->addWidget(opt);
+
 	a = new QHBoxLayout;	o->addLayout(a);
 	b = new QPushButton(tr("Cancel"),this);	a->addWidget(b);
 	connect(b, SIGNAL(clicked()), this, SLOT(reject()));
@@ -87,14 +84,14 @@ NewCmdDialog::NewCmdDialog(QWidget *p) : QDialog(p,Qt::WindowStaysOnTopHint)
 	a = new QHBoxLayout;	o->addLayout(a);
 	help = new QTextBrowser(this);		help->setMinimumWidth(500);
 	help->setOpenExternalLinks(false);	o->addWidget(help);
-	t = new QToolButton(p);	t->setIcon(QPixmap(":/xpm/go-previous.png"));
+	t = new QToolButton(p);	t->setIcon(QPixmap(":/png/go-previous.png"));
 	connect(t, SIGNAL(clicked()), help, SLOT(backward()));	a->addWidget(t);
-	t = new QToolButton(p);	t->setIcon(QPixmap(":/xpm/go-next.png"));
+	t = new QToolButton(p);	t->setIcon(QPixmap(":/png/go-next.png"));
 	connect(t, SIGNAL(clicked()), help, SLOT(forward()));	a->addWidget(t);
 	a->addStretch(1);
-	t = new QToolButton(p);	t->setIcon(QPixmap(":/xpm/zoom-in.png"));
+	t = new QToolButton(p);	t->setIcon(QPixmap(":/png/zoom-in.png"));
 	connect(t, SIGNAL(clicked()), this, SLOT(zoomIn()));	a->addWidget(t);
-	t = new QToolButton(p);	t->setIcon(QPixmap(":/xpm/zoom-out.png"));
+	t = new QToolButton(p);	t->setIcon(QPixmap(":/png/zoom-out.png"));
 	connect(t, SIGNAL(clicked()), this, SLOT(zoomOut()));	a->addWidget(t);
 
 	connect(type, SIGNAL(currentIndexChanged(int)),this,SLOT(typeChanged(int)));
@@ -116,12 +113,13 @@ void NewCmdDialog::parseCmd(const QString &txt)
 	QRegExp sep("[ \t]");
 	QString cmd = str.section(sep,0,0),a,b;
 	bool opt,var,chr;
+	replace = false;
 	for(int i=0;i<17;i++)
 	{
 		if(cmds[i].contains(cmd))	// find command first
 		{
 			typeChanged(i);
-			nameChanged(cmds[i].indexOf(cmd));
+			name->setCurrentIndex(cmds[i].indexOf(cmd));
 			register int j,j0,k,k0;
 			bool ok;
 			for(j0=k0=-1,j=0;j<NUM_CH;j++)		// determine set of arguments
@@ -141,22 +139,47 @@ void NewCmdDialog::parseCmd(const QString &txt)
 			}
 			if(j0>=0)	// best choice
 			{
-				kindChanged(j0);
+				kind->setCurrentIndex(j0);
 				for(k=0;k<argn[j0].count();k++)
 					args->item(k,1)->setText(str.section(sep,k+1,k+1).trimmed());
 			}
-			return;		// selection is done
+			replace = true;	return;		// selection is done
 		}
 	}
 }
 //-----------------------------------------------------------------------------
+#include "xpm/preview.xpm"
+#include "xpm/text.xpm"
+#include "xpm/table.xpm"
+#include "xpm/oper_dir.xpm"
+#include "xpm/axis.xpm"
+#include "xpm/axis_sh.xpm"
+#include "xpm/plot.xpm"
+#include "xpm/box.xpm"
+#include "xpm/curve.xpm"
+#include "xpm/other.xpm"
+#include "xpm/vect.xpm"
+#include "xpm/tiles.xpm"
+//-----------------------------------------------------------------------------
 void NewCmdDialog::fillList()
 {
-	types<<tr("1D plots")<<tr("2D plots")<<tr("3D plots")<<tr("Dual plots")
-			<<tr("Vector plots")<<tr("Other plots")<<tr("Text and legend")
-			<<tr("Create data and I/O")<<tr("Data transform")<<tr("Data handling")
-			<<tr("Axis and colorbar")<<tr("Axis setup")<<tr("General setup")
-			<<tr("Scale and rotate")<<tr("Program flow")<<tr("Primitives");
+	type->addItem(QPixmap(plot_xpm), tr("1D plots"));
+	type->addItem(QPixmap(preview_xpm), tr("2D plots"));
+	type->addItem(QPixmap(":/png/weather-clouds.png"), tr("3D plots"));
+	type->addItem(QPixmap(tiles_xpm), tr("Dual plots"));
+	type->addItem(QPixmap(vect_xpm), tr("Vector plots"));
+	type->addItem(QPixmap(other_xpm), tr("Other plots"));
+	type->addItem(QPixmap(text_xpm), tr("Text and legend"));
+	type->addItem(QPixmap(table_xpm), tr("Create data and I/O"));
+	type->addItem(QPixmap(oper_dir_xpm), tr("Data transform"));
+	type->addItem(QPixmap(oper_dir_xpm), tr("Data handling"));
+	type->addItem(QPixmap(axis_xpm), tr("Axis and colorbar"));
+	type->addItem(QPixmap(axis_sh_xpm), tr("Axis setup"));
+	type->addItem(QPixmap(":/png/preferences-system.png"), tr("General setup"));
+	type->addItem(QPixmap(box_xpm), tr("Scale and rotate"));
+	type->addItem(QPixmap(":/png/media-playback-start.png"), tr("Program flow"));
+	type->addItem(QPixmap(curve_xpm), tr("Primitives"));
+
 	// now fill it automatically from parser for all categories
 	long i, n = parser.GetCmdNum();
 	for(i=0;i<n;i++)
@@ -241,8 +264,7 @@ void NewCmdDialog::nameChanged(int s)
 		kinds<<n+" "+a;
 		parse(argn[k],a);
 	}
-	name->setCurrentIndex(s);
-	kind->addItems(kinds);	kind->setCurrentIndex(0);
+	kind->addItems(kinds);	kind->setCurrentIndex(0);	replace = false;
 }
 //-----------------------------------------------------------------------------
 void NewCmdDialog::kindChanged(int s)
@@ -291,7 +313,20 @@ void NewCmdDialog::insertData()
 	{
 		if(datDialog->exec())	args->item(row,1)->setText(datDialog->getData());
 	}
-	else		QMessageBox::warning(this,tr("New command"), tr("This argument is not data"));
+	else if(a=="'fmt'" || a=="_'fmt'")
+	{
+		if(stlDialog->exec())	args->item(row,1)->setText(stlDialog->getStyle());
+	}
+	else if(a=="'file'")
+	{
+		QString str = QFileDialog::getOpenFileName(this, tr("UDAV - Insert filename"));
+		if(!str.isEmpty())	args->item(row,1)->setText(str);
+	}
+	else if(a=="'path'")
+	{
+		QString str = QFileDialog::getExistingDirectory(this, tr("UDAV - Insert path"));
+		if(!str.isEmpty())	args->item(row,1)->setText(str);
+	}
 }
 //-----------------------------------------------------------------------------
 void NewCmdDialog::insertStl()
@@ -308,8 +343,7 @@ void NewCmdDialog::insertStl()
 	int i;
 	i = argn[s].indexOf("'fmt'");
 	if(i<0)	i = argn[s].indexOf("_'fmt'");
-	if(!stlDialog->exec())	return;
-	args->item(i,1)->setText(stlDialog->getStyle());
+	if(!stlDialog->exec())	args->item(i,1)->setText(stlDialog->getStyle());
 }
 //-----------------------------------------------------------------------------
 void NewCmdDialog::finish()
@@ -352,6 +386,6 @@ void NewCmdDialog::finish()
 			cmd = cmd + ' ' + txt;
 		}
 	}
-	cmd = cmd + opt->text();	accept();	emit result(cmd);
+	cmd = cmd + opt->text();	accept();	emit result(cmd, replace);
 }
 //-----------------------------------------------------------------------------
