@@ -18,10 +18,9 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <locale.h>
-#include <unistd.h>
 #include "mgl2/mgl.h"
 //-----------------------------------------------------------------------------
-int mgl_hex(char ch)
+int MGL_LOCAL_CONST mgl_hex(char ch)
 {
 	int res=-1;
 	if(ch>='0' && ch<='9')	res = ch-'0';
@@ -54,6 +53,7 @@ void mgl_get_value(const char *buf, const char *name, char *val)
 //-----------------------------------------------------------------------------
 int main()
 {
+	mgl_suppress_warn(true);
 	mglGraph gr;
 	mglParse p(true);
 
@@ -68,7 +68,7 @@ int main()
 	{
 		long len=atol(getenv("CONTENT_LENGTH"));
 		buf = new char[len+1];
-		fread(buf,len,1,stdin);
+		if(!fread(buf,len,1,stdin))	len=0;
 		buf[len]=0;	alloc=true;
 	}
 	else		buf = getenv("QUERY_STRING");
