@@ -22,7 +22,6 @@
 #include <FL/Fl_Double_Window.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_File_Chooser.H>
-//#include <unistd.h>
 //-----------------------------------------------------------------------------
 #ifdef USE_GETTEXT
 #include <libintl.h>
@@ -34,7 +33,7 @@
 #endif
 //-----------------------------------------------------------------------------
 #include "mgl2/canvas_wnd.h"
-#include "mgl2/fltk.h"
+#include "mgl2/Fl_MathGL.h"
 //-----------------------------------------------------------------------------
 #include "xpm/show_sl.xpm"
 #include "xpm/next_sl.xpm"
@@ -53,6 +52,7 @@
 #include "xpm/ok.xpm"
 #include "xpm/wire.xpm"
 #include "xpm/stop.xpm"
+#include "xpm/pause.xpm"
 //-----------------------------------------------------------------------------
 Fl_Pixmap xpm_a1(alpha_xpm);
 Fl_Pixmap xpm_l1(light_xpm);
@@ -60,6 +60,7 @@ Fl_Pixmap xpm_z1(zoom_in_xpm);
 Fl_Pixmap xpm_s1(show_sl_xpm);
 Fl_Pixmap xpm_r1(rotate_xpm);
 Fl_Pixmap xpm_wire(wire_xpm);
+Fl_Pixmap xpm_pause(pause_xpm);
 //-----------------------------------------------------------------------------
 /// Class allows the window creation for displaying plot bitmap with the help of FLTK library
 /** NOTE!!! All frames are saved in memory. So animation with many frames require a lot memory and CPU time (for example, for mouse rotation).*/
@@ -127,7 +128,7 @@ void Fl_MathGL::set_graph(HMGL GR)
 {
 	mglCanvas *gg = dynamic_cast<mglCanvas *>(GR);
 	if(!gg)	return;
-	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
+	if(gr && mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	gr=gg;	mgl_use_graph(gg,1);
 	gr->SetEventFunc(mgl_fltk_event_func, NULL);
 }
@@ -335,9 +336,31 @@ void Fl_MGLView::setoff(int &val, Fl_Button *b, const char *txt)
 	//update();
 }
 //-----------------------------------------------------------------------------
+void Fl_MGLView::exec_pause()
+{
+#if MGL_HAVE_PTHR_WIDGET
+	pthread_mutex_t *mutex=0;
+	mglCanvasWnd *g=dynamic_cast<mglCanvasWnd *>(FMGL->gr);
+	if(g && g->mutex)	mutex = g->mutex;
+	else
+	{
+		mglDraw *d=FMGL->get_class();
+		if(d)	mutex = &(d->mutex);
+	}
+	if(mutex)
+	{
+		if(pauseC)	pthread_mutex_lock(mutex);
+		else	pthread_mutex_unlock(mutex);
+	}
+#endif
+}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_pause_cb(Fl_Widget*, void* v)
+{	if(v)	((Fl_MGLView*)v)->toggle_pause();	}
+//-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_grid_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->toggle_grid();	}
-//-------------------------------------------------------------------------
+//-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_alpha_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->toggle_alpha();	}
 void mglCanvasFL::ToggleAlpha()	{	mgl->toggle_alpha();	}
@@ -378,7 +401,7 @@ void Fl_MGLView::update()
 }
 void MGL_NO_EXPORT mgl_draw_cb(Fl_Widget*, void* v)
 {	if(v)	((Fl_MGLView*)v)->update();	}
-void mglCanvasFL::Update()		{	mgl->update();	}
+void mglCanvasFL::Update()		{	mgl->update();	Wnd->show();	}
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_export_png_cb(Fl_Widget*, void* v)
 {
@@ -574,7 +597,7 @@ Fl_Menu_Item pop_graph[20] = {
 Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Window(xx,yy,ww,hh,lbl)
 {
 	Fl_Button *o;
-	grid = alpha = light = sshow = 0;	menu = 0;
+	grid = alpha = light = sshow = pauseC = 0;	menu = 0;
 	next = prev = reload = NULL;	delay = NULL;
 
 	Fl_Group *g = new Fl_Group(0,0,435,30);
@@ -610,7 +633,7 @@ Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Win
 	phi->tooltip(mgl_gettext("Phi angle (rotate in x*y plane)"));
 	g->end();	g->resizable(0);
 
-	g = new Fl_Group(0,0,30,260);
+	g = new Fl_Group(0,0,30,285);
 	o = new Fl_Button(1, 30, 25, 25);		o->tooltip(mgl_gettext("Shift the picture up"));
 	o->image(new Fl_Pixmap(up_1_xpm));		o->callback(mgl_su_cb,this);
 //	o->box(FL_PLASTIC_UP_BOX);	o->down_box(FL_PLASTIC_DOWN_BOX);
@@ -625,6 +648,7 @@ Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Win
 	o = new Fl_Button(1, 155, 25, 25);		o->tooltip(mgl_gettext("Shift the picture down"));
 	o->image(new Fl_Pixmap(down_1_xpm));	o->callback(mgl_sd_cb,this);
 
+
 	o = new Fl_Button(1, 185, 25, 25);		o->tooltip(mgl_gettext("Show previous frame in slideshow"));
 	o->image(new Fl_Pixmap(prev_sl_xpm));	o->callback(mgl_sprev_cb,this);
 	anim_bt = new Fl_Button(1, 210, 25, 25);	anim_bt->type(FL_TOGGLE_BUTTON);
@@ -632,6 +656,13 @@ Fl_MGLView::Fl_MGLView(int xx, int yy, int ww, int hh, const char *lbl) : Fl_Win
 	anim_bt->tooltip(mgl_gettext("Run/Stop slideshow (graphics animation)"));
 	o = new Fl_Button(1, 235, 25, 25);		o->tooltip(mgl_gettext("Show next frame in slideshow"));
 	o->image(new Fl_Pixmap(next_sl_xpm));	o->callback(mgl_snext_cb,this);
+
+#if MGL_HAVE_PTHR_WIDGET
+	pause_bt = new Fl_Button(1, 260, 25, 25);	pause_bt->type(FL_TOGGLE_BUTTON);
+	pause_bt->image(xpm_pause);	pause_bt->callback(mgl_pause_cb,this);
+	pause_bt->tooltip(mgl_gettext("Pause on/off calculations"));
+#endif
+
 	g->end();	g->resizable(0);
 
 	scroll = new Fl_Scroll(30, 30, 800, 600);
@@ -649,7 +680,7 @@ Fl_MGLView::~Fl_MGLView()	{}
 //
 //-----------------------------------------------------------------------------
 mglCanvasFL::mglCanvasFL() : mglCanvasWnd()	{	Wnd=0;	}
-mglCanvasFL::~mglCanvasFL()		{	if(Wnd)	delete Wnd;	}
+mglCanvasFL::~mglCanvasFL()		{	if(Wnd)	{	mgl->FMGL->gr=0;	delete Wnd;	}	}
 //-----------------------------------------------------------------------------
 void mglCanvasFL::GotoFrame(int d)
 {
@@ -676,6 +707,7 @@ void MGL_EXPORT mgl_makemenu_fltk(Fl_Menu_ *m, Fl_MGLView *w)
 	m->add("Graphics/Reload data", "f9", mgl_oncemore_cb, w);
 	m->add("Graphics/Stop", "f7", mgl_stop_cb, w);
 	//TODO	m->add("Graphics/Copy graphics","+^c", mgl_copyimg_cb, w);
+	m->add("Graphics/Pause calc", "^t", mgl_pause_cb, w, FL_MENU_TOGGLE);
 
 	m->add("Graphics/Export/as PNG", "#p", mgl_export_png_cb, w);
 	m->add("Graphics/Export/as solid PNG", "#f", mgl_export_pngn_cb, w);
@@ -747,7 +779,7 @@ MGL_NO_EXPORT void *mgl_fltk_tmp(void *)
 //-----------------------------------------------------------------------------
 int MGL_EXPORT mgl_fltk_thr()		// NOTE: Qt couldn't be running in non-primary thread
 {
-#if MGL_HAVE_PTHREAD
+#if MGL_HAVE_PTHR_WIDGET
 	static pthread_t thr;
 	pthread_create(&thr,0,mgl_fltk_tmp,0);
 	pthread_detach(thr);
